/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): EBM WebSourcing
 * --------------------------------------------------------------------------
 * $Id: MX4JRMIAdaptor.java,v 1.2 2005/07/22 10:24:27 ddesjardins Exp $
 * --------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jmx.rmi;

import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;

import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;
import javax.management.remote.rmi.RMIConnectorServer;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.jmx.agent.AdminAttributes;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * This is the Fractal JMX adaptor for use with MX4J tools.
 *
 * @author ddesjardins - eBMWebsourcing
 */
@FractalComponent
public class RMIConnector {

    /**
     * AdminAttributes
     */
	@Requires(name="adminAtt",signature=org.objectweb.fractal.jmx.agent.AdminAttributes.class)
    protected AdminAttributes adminAttributes;

    /**
     * Logger
     */
    protected LoggingUtil log;

    /**
     * Logger
     */
    @Monolog(name="logger")
    protected Logger logger;

    /**
     * JMX server
     */
    protected JMXConnectorServer cs;

    /**
     * Start the JMX RMI Server
     */
    @LifeCycle(on=LifeCycleType.START)
    public void start() {
        log=new LoggingUtil(null);
    	log.start();
        try {
            // Retreive the ports and host properties
            int port = Integer.parseInt(SystemUtil.getJmxJndiPort());
            String host = SystemUtil.getHost();
            int jmxPort = Integer.parseInt(SystemUtil.getJmxPort());
            if (cs != null && cs.isActive()
                && cs.getAddress().getPort() == port) {
                return;
            }
            if (cs != null) {
                cs.stop();
            }
            // create rmi connectors
            Registry registry = LocateRegistry.getRegistry(port);
            try {
                registry.list();
            } catch (Exception e) {
                LocateRegistry.createRegistry(port);
            }

            JMXServiceURL jmxUrl = new JMXServiceURL(getJmxServiceURL(host, jmxPort, port));
            // Server with security (need keystore that are not yet done)
            // HashMap<String, Object> env = new HashMap<String, Object>();
            // env.put("jmx.remote.x.password.file", "conf" + File.separator
            // + "password.properties");
            // env.put("jmx.remote.x.access.file", "conf" + File.separator
            // + "access.properties");
            // RMIConnectorServer connectorServer = (RMIConnectorServer)
            // JMXConnectorServerFactory
            // .newJMXConnectorServer(jmxUrl, env, beanServer);

            // Server without security
            RMIConnectorServer connectorServer = (RMIConnectorServer) JMXConnectorServerFactory
                .newJMXConnectorServer(jmxUrl, null, adminAttributes
                    .getRawMBeanServer());
            cs = connectorServer;
            // Start the server
            cs.start();
            // Write the address of the server on the console
            log.info("JMX RMI server started at : "
                + cs.getAddress().getURLPath());
        } catch (Exception e) {
            log.error(
                "Error during the instanciation of the MX4J RMI adaptor:", e);
        }
        log.end();
    }
    
    /**
     * Retrieve the JmxServiceURL corresponding to the host, jmxPort
     * and jmxJndiPort passed as arguments.<br>
     * If jmxPort == -1, it is retrieved dynamically.
     * @param host the jmx host
     * @param jmxPort the jmx port
     * @param jmxJndiPort the jmx jndi port
     * @return
     */
    public static final String getJmxServiceURL(String host, int jmxPort, int jmxJndiPort) {
        return getJmxServiceURL(host, ""+jmxPort, ""+jmxJndiPort);
    }
    
    /**
     * Retrieve the JmxServiceURL corresponding to the host, jmxPort
     * and jmxJndiPort passed as arguments.<br>
     * If jmxPort is not set (!= '-1'), it is retrieved dynamically.
     * @param host the jmx host
     * @param jmxPort the jmx port
     * @param jmxJndiPort the jmx jndi port
     * @return
     */
    public static final String getJmxServiceURL(String host, String jmxPort, String jmxJndiPort) {
        StringBuffer sJmxURL = new StringBuffer("service:jmx:rmi://");
        // We test if jmxPort is set. Otherwise, it is set dynamically
        if (jmxPort != null && ! "-1".equals(jmxPort)) {
            sJmxURL.append(host).append(":").append(jmxPort);
        }
        sJmxURL.append("/jndi/rmi://").append(host).append(":").append(jmxJndiPort).append("/management/rmi-jmx-connector");
        return sJmxURL.toString();
    }

    /**
     * Stop the RMI Server
     */
    @LifeCycle(on=LifeCycleType.STOP)
    public void stop() {
        log.start();
        if (cs != null) {
            try {
                cs.stop();
            } catch (Exception e) {
                log.error("Problem while stopping the JMX server", e);
            }
        }
        log.end();
    }

}
