/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.communication.jndi;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * Fractal component used to retrieve the JNDI initial context.
 * 
 * @author nsalatge - eBM WebSourcing
 * 
 */
public final class JNDIEndpointCache {

	/**
	 * the singleton.
	 */
	private final static JNDIEndpointCache singleton;

	/**
	 * the endpointsCache.
	 */
	private Map<String, Object> endpointsCache = null;
	
	static {
	   singleton = new JNDIEndpointCache();
	}

	/**
	 * the private constructor.
	 */
	private JNDIEndpointCache() {
		this.endpointsCache = new ConcurrentHashMap<String, Object>();
	}

	/**
	 * get the instance.
	 * 
	 * @return the instance
	 */
	public static final JNDIEndpointCache getInstance() {
		return JNDIEndpointCache.singleton;
	}

	/**
	 * Get an entry from the cache.
	 * 
	 * @return the cache
	 */
	public Object getEntry(String name) {
		return this.endpointsCache.get(name);
	}

	/**
	 * Put an entry to the cache.
	 * 
	 * @return the cache
	 */
	public void putEntry(String name, Object value) {
		this.endpointsCache.put(name, value);
	}
	
	/**
	 * Remove an entry from the cache.
	 * 
	 * @return the cache
	 */
	public void removeEntry(String name) {
		this.endpointsCache.remove(name);
	}

	/**
	 * clean the cache.
	 * 
	 */
	public void cleanCache() {
		this.endpointsCache.clear();
	}

}
