/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jndi.joram;

import java.io.File;
import java.io.FileNotFoundException;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.jndi.AbstractJNDIAgentService;
import org.objectweb.petals.jbi.messaging.transport.util.JoramControl;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

import fr.dyade.aaa.agent.AgentServer;

/**
 * The implementation of the JORAM JNDI agent
 *
 * @author Christophe HAMERLING - eBMWebSourcing
 *
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.communication.jndi.JNDIAgentService.class))
public class JoramJNDIAgentServiceImpl extends AbstractJNDIAgentService {

    /**
     * Get JORAM controller
     */
    private JoramControl joramControl = JoramControl.getInstance();

    /**
     * The directory used by the JNDI server
     */
    protected File jndiDir;

    /*
     * TODO : there is a bug when the logger is defined in the mother abstract class.
     * Move back the method in AbstractJNDIAgentService when fixed.
     */
    /**
     * Logger
     */
    @Monolog(name = "logger")
    protected Logger logger;

    @LifeCycle(on = LifeCycleType.START)
    protected void start() throws Exception {
        log = new LoggingUtil(logger);
        log.start();
        status = Status.NOT_INITIALIZED;
        dynamic = SystemUtil.isDynamicTopology();

        if (!dynamic) {
            host = SystemUtil.getHost();
            port = Integer.parseInt(SystemUtil.getJndiPort());
        }

        jndiDir = new File(SystemUtil.getJoramPath());
        if (!jndiDir.exists()) {
            jndiDir.mkdirs();
        }

        // initialize and start the JNDI server
        initJNDIAgent();
        startJNDIAgent();
        log.end();
    }

    @LifeCycle(on = LifeCycleType.STOP)
    protected void stop() throws Exception {
        log.start();
        try {
            stopJNDIAgent();
        } catch (PetalsException e) {
            log.error(getImplName() + " : Can not stop the JNDI component");
            throw new PetalsException(getImplName()
                + " : Can not stop the JNDI component", e);
        }
        log.end();
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.jndi.AbstractJNDIAgentService#doInit()
     */
    @Override
    public void doInit() throws Exception {
        if (!dynamic) {

            // The configuration file must be present to start JORAM
            File configFile = new File(jndiDir, AgentServer.DEFAULT_CFG_FILE);
            if (!configFile.exists()) {
                throw new FileNotFoundException(
                    "JORAM configuration file has not been found.");
            }

            // initialize JORAM with system properties
            String id = SystemUtil.getJoramId();
            joramControl.initAgentServer(jndiDir, Short.parseShort(id));
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.jndi.AbstractJNDIAgentService#doStart()
     */
    @Override
    public void doStart() throws Exception {
        if (!dynamic) {
            joramControl.startAgentServer();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.jndi.AbstractJNDIAgentService#doStop()
     */
    @Override
    public void doStop() throws Exception {
        if (!dynamic) {
            joramControl.stopAgentServer();
        }
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.communication.jndi.AbstractJNDIAgentService#getImplName()
     */
    public String getImplName() {
        return "JORAM";
    }
}
