/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDICommand.java 7:04:52 PM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jndi.tribe.msg.request;

import java.io.Serializable;

/**
 * This class is a registry request which is used to send request to the
 * registry
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public class RegistryRequest implements Serializable {

    /**
     * Request types
     * 
     * @author ddesjardins - eBMWebsourcing
     */
    public static enum RequestType {
        /**
         * 
         */
        addToEnvironment,
        /**
         * BIND : arg1 = context name, arg2 = key, arg3 = value
         */
        bind,
        /**
         * CREATE SUBCONTEXT : arg1 = context name, arg2 = name of the context
         * to create
         */
        createSubcontext,
        /**
         * DESTROY SUBCONTEXT : arg1 = context name, arg2 = nam of the context
         * to destroy
         */
        destroySubcontext,
        /**
         * FULL UPDATE : no args
         */
        fullUpdate,
        /**
         * LIST : arg1 = context name, arg2 = context name to list the content
         */
        list,
        /**
         * LIST BINDINGS : arg1 = context name, arg2 = context name to list the
         * content
         */
        listBindings,
        /**
         * LOOKUP : arg1 = context name, arg2 = key
         */
        lookup,
        /**
         * LOOKUP LINK : arg1 = context name, arg2 = key
         */
        lookupLink,
        /**
         * NEW MASTER : no args
         */
        newMaster,
        /**
         * PING : no args
         */
        ping,
        /**
         * REBIND : arg1 = context name, arg2 = key, arg3 = value
         */
        rebind,
        /**
         * RENAME : arg1 = context name, arg2 = old key, arg3 = new key
         */
        rename,
        /**
         * UNBIND : arg1 = context name, arg2 = key
         */
        unbind
    }

    private static final long serialVersionUID = 1L;

    /**
     * First argument
     */
    protected Object arg1;

    /**
     * Second argument
     */
    protected Object arg2;

    /**
     * Third argument
     */
    protected Object arg3;

    /**
     * Start time of the recipient or 0 to send it to everybody
     */
    protected long recipient;

    /**
     * Date of the request
     */
    protected long requestDate;

    /**
     * Start time of the sender
     */
    protected long sender;

    /**
     * Type of request
     */
    protected RequestType type;

    protected RegistryRequest(RequestType type, long sender, long recipient) {
        super();
        this.type = type;
        this.sender = sender;
        this.recipient = recipient;
        requestDate = System.nanoTime();
    }

    protected RegistryRequest(RequestType type, Object arg1, long sender,
        long recipient) {
        super();
        this.type = type;
        this.arg1 = arg1;
        this.sender = sender;
        this.recipient = recipient;
        requestDate = System.nanoTime();
    }

    protected RegistryRequest(RequestType type, Object arg1, Object arg2,
        long sender, long recipient) {
        super();
        this.type = type;
        this.arg1 = arg1;
        this.arg2 = arg2;
        this.sender = sender;
        this.recipient = recipient;
        requestDate = System.nanoTime();
    }

    protected RegistryRequest(RequestType type, Object arg1, Object arg2,
        Object arg3, long sender, long recipient) {
        super();
        this.type = type;
        this.arg1 = arg1;
        this.arg2 = arg2;
        this.arg3 = arg3;
        this.sender = sender;
        this.recipient = recipient;
        requestDate = System.nanoTime();
    }

    public Object getArg1() {
        return arg1;
    }

    public Object getArg2() {
        return arg2;
    }

    public Object getArg3() {
        return arg3;
    }

    public long getRecipient() {
        return recipient;
    }

    public long getRequestDate() {
        return requestDate;
    }

    public long getSender() {
        return sender;
    }

    public RequestType getType() {
        return type;
    }

    public void setArg1(Object arg1) {
        this.arg1 = arg1;
    }

    public void setArg2(Object arg2) {
        this.arg2 = arg2;
    }

    public void setArg3(Object arg3) {
        this.arg3 = arg3;
    }

    public void setRecipient(long recipient) {
        this.recipient = recipient;
    }

    public void setSender(long sender) {
        this.sender = sender;
    }

    public void setType(RequestType type) {
        this.type = type;
    }

    public String toString() {
        return "RegistryRequest " + type + " date " + requestDate;
    }

}
