/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JNDIResponse.java 10:55:58 AM ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jndi.tribe.msg.response;

import java.io.Serializable;

/**
 * Registry response
 *
 * @author ddesjardins - eBMWebsourcing
 */
public class RegistryResponse implements Serializable {

    /**
     * Types of response
     *
     * @author ddesjardins - eBMWebsourcing
     */
    public static enum ResponseType {
        /*
         * ACK :
         */
        ack,
        /*
         * CREATE SUBCONTEXT : arg1 = name of the newly created context
         */
        createSubcontext,
        /*
         * EXCEPTION : arg1 = naming exception
         */
        exception,
        /*
         * FULL UPDATE : arg1 = full data
         */
        fullUpdate,
        /*
         * LIST : arg1 = array of NameClassPair
         */
        list,
        /*
         * LIST BINDINGS : arg1 = array of Binding
         */
        listBindings,
        /*
         * LOOKUP : arg1 = value of the lookup object
         */
        lookup,
        /*
         * LOOKUP LINK: arg1 = value of the lookup object
         */
        lookupLink,
        /*
         * MY START TIME : arg1 : start time
         */
        myStartTime
    }

    private static final long serialVersionUID = 1L;;

    /**
     * First argument of the response
     */
    protected Object arg1;

    /**
     * Start time of the recipient for a specific recipient, 0 for everybody
     */
    protected long recipient;

    /**
     * Response date
     */
    protected long responseDate;

    /**
     * Start time of the sender
     */
    protected long sender;

    /**
     * Response type
     */
    protected ResponseType type;

    /**
     * Creates a new instance of {@link RegistryResponse}
     *
     * @param type
     * @param sender
     * @param recipient
     */
    protected RegistryResponse(ResponseType type, long sender, long recipient) {
        super();
        this.type = type;
        this.sender = sender;
        this.recipient = recipient;
        responseDate = System.nanoTime();
    }

    /**
     * Creates a new instance of {@link RegistryResponse}
     *
     * @param type
     * @param arg1
     * @param sender
     * @param recipient
     */
    protected RegistryResponse(ResponseType type, Object arg1, long sender,
        long recipient) {
        super();
        this.type = type;
        this.arg1 = arg1;
        this.sender = sender;
        this.recipient = recipient;
        responseDate = System.nanoTime();
    }

    public Object getArg1() {
        return arg1;
    }

    public long getRecipient() {
        return recipient;
    }

    public long getResponseDate() {
        return responseDate;
    }

    public long getSender() {
        return sender;
    }

    public ResponseType getType() {
        return type;
    }

    public void setArg1(Object arg1) {
        this.arg1 = arg1;
    }

    public void setRecipient(int recipient) {
        this.recipient = recipient;
    }

    public void setSender(int sender) {
        this.sender = sender;
    }

    public void setType(ResponseType type) {
        this.type = type;
    }

    public String toString() {
        return "RegistryResponse " + type + " date " + responseDate;
    }

}
