/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ContainerInformation.java 16:40:52 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.network;

import java.io.Serializable;
import java.util.Properties;

/**
 * This class contains all the informations of a container.
 *
 * @author ddesjardins - eBMWebsourcing
 * @author chamerling - eBM WebSourcing
 */
public class ContainerInformation implements Serializable {

    public static final String UNUSED = "unused";
    
    /**
     * Container state
     *
     * @author ddesjardins - eBMWebsourcing
     */
    public static enum ContainerState {
        /**
         * Started state
         */
        STARTED,
        /**
         * Stopped state
         */
        STOPPED,

        /**
         * unknown state
         */
        UNKNOWN
    };

    /**
     * the "unique" ID for serialization
     */
    private static final long serialVersionUID = 1L;    
    

    /**
     * Port of the Dream TCP
     */
    private String dreamTCPPort;    
    

    /**
     * Host of the petals container
     */
    private String host;

    /**
     * Port of the html adaptor
     */
    private String htmlPort;

    /**
     * Login to connect to the JMX server
     */
    private String jmxLogin;

    /**
     * Password to connect to the JMX server
     */
    private String jmxPassword;

    /**
     * Port of the JMX JNDI server
     */
    private String jmxJndiPort;

    /**
     * Port of the JMX server
     */
    private String jmxPort;

    /**
     * JNDI Factory
     */
    private String jndiFactory;

    /**
     * JNDI port
     */
    private String jndiPort;

    /**
     * The name of the used jndi server
     */
    private String jndiServer;

    /**
     * Name of the Joram domain
     */
    private String joramDomain;

    /**
     * Port of the Joram domain
     */
    private String joramDomainPort;

    /**
     * Id of the Joram server
     */
    private String joramId;

    /**
     * Login to connect to the Joram server
     */
    private String joramLogin;

    /**
     * Password to connect to the Joram server
     */
    private String joramPassword;

    /**
     * Port of the Joram TCP topic
     */
    private String joramTCPPort;

    /**
     * Container name
     */
    private String name;

    /**
     * State of the container
     */
    private ContainerState state = ContainerState.UNKNOWN;

    /**
     * Container uid
     */
    private long uid;

	/**
	 * All properties defined in the server.xml of the container used to store
	 * all paremeter values of PEtALS extensions.
	 */
    private Properties properties;

    public Properties getProperties() {
    	return properties;
    }
    
    public void setProperties(final Properties properties) {
    	this.properties = properties;
    }

    /**
     * Get the dream TCP port.
     * @return the value of the port
     */
    public String getDreamTCPPort() {
      return dreamTCPPort;
    }        
        
    /**
     *
     * @return
     */
    public String getHost() {
        return host;
    }

    /**
     *
     * @return
     */
    public String getHtmlPort() {
        return htmlPort;
    }

    /**
     *
     * @return
     */
    public String getJmxLogin() {
        return jmxLogin;
    }

    /**
     *
     * @return
     */
    public String getJmxPassword() {
        return jmxPassword;
    }

    /**
     *
     * @return
     */
    public String getJmxPort() {
        return jmxPort;
    }

    /**
     *
     * @return
     */
    public String getJoramDomain() {
        return joramDomain;
    }

    /**
     *
     * @return
     */
    public String getJoramDomainPort() {
        return joramDomainPort;
    }

    /**
     *
     * @return
     */
    public String getJoramId() {
        return joramId;
    }

    /**
     *
     * @return
     */
    public String getJoramLogin() {
        return joramLogin;
    }

    /**
     *
     * @return
     */
    public String getJoramPassword() {
        return joramPassword;
    }

    /**
     *
     * @return
     */
    public String getJoramTCPPort() {
        return joramTCPPort;
    }

    /**
     *
     * @return
     */
    public String getName() {
        return name;
    }

    /**
     *
     * @return
     */
    public ContainerState getState() {
        return state;
    }

    /**
     *
     * @return
     */
    public long getUid() {
        return uid;
    }

    /**
     *
     * @return
     */
    public boolean isStarted() {
        return state.equals(ContainerState.STARTED);
    }

    /**
     *
     * @return
     */
    public boolean isStopped() {
        return state.equals(ContainerState.STOPPED);
    }

    /**
     * Set the dream TCP Port
     * @param dreamTCPPort the value of the port.
     */
    public void setDreamTCPPort(String dreamTCPPort) {
      this.dreamTCPPort = dreamTCPPort;
    }        
    
    /**
     *
     * @param host
     */
    public void setHost(String host) {
        this.host = host;
    }

    /**
     *
     * @param htmlPort
     */
    public void setHtmlPort(String htmlPort) {
        this.htmlPort = htmlPort;
    }

    /**
     *
     * @param jmxLogin
     */
    public void setJmxLogin(String jmxLogin) {
        this.jmxLogin = jmxLogin;
    }

    /**
     *
     * @param jmxPassword
     */
    public void setJmxPassword(String jmxPassword) {
        this.jmxPassword = jmxPassword;
    }

    /**
     *
     * @param jmxPort
     */
    public void setJmxPort(String jmxPort) {
        this.jmxPort = jmxPort;
    }

    /**
     *
     * @param joramDomain
     */
    public void setJoramDomain(String joramDomain) {
        this.joramDomain = joramDomain;
    }

    /**
     *
     * @param joramPort
     */
    public void setJoramDomainPort(String joramPort) {
        this.joramDomainPort = joramPort;
    }

    /**
     *
     * @param joramId
     */
    public void setJoramId(String joramId) {
        this.joramId = joramId;
    }

    /**
     *
     * @param joramLogin
     */
    public void setJoramLogin(String joramLogin) {
        this.joramLogin = joramLogin;
    }

    /**
     *
     * @param joramPassword
     */
    public void setJoramPassword(String joramPassword) {
        this.joramPassword = joramPassword;
    }

    /**
     *
     * @param joramTCPPort
     */
    public void setJoramTCPPort(String joramTCPPort) {
        this.joramTCPPort = joramTCPPort;
    }

    /**
     *
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     *
     * @param state
     */
    public void setState(ContainerState state) {
        this.state = state;
    }

    /**
     *
     * @param uid
     */
    public void setUid(long uid) {
        this.uid = uid;
    }

    /**
     * @return the jndiFactory
     */
    public String getJndiFactory() {
        return jndiFactory;
    }

    /**
     * @param jndiFactory
     *            the jndiFactory to set
     */
    public void setJndiFactory(String jndiFactory) {
        this.jndiFactory = jndiFactory;
    }

    /**
     * @return the jndiPort
     */
    public String getJndiPort() {
        return jndiPort;
    }

    /**
     * @param jndiPort
     *            the jndiPort to set
     */
    public void setJndiPort(String jndiPort) {
        this.jndiPort = jndiPort;
    }

    /**
     * @return the jndiServer
     */
    public String getJndiServer() {
        return jndiServer;
    }

    /**
     * @param jndiServer
     *            the jndiServer to set
     */
    public void setJndiServer(String jndiServer) {
        this.jndiServer = jndiServer;
    }

    /*
     * (non-Javadoc)
     *
     * @see java.lang.Object#toString()
     */
    public String toString() {
        return "Container " + name + " (State " + state + ") host: "
            + host + " Joram Id/domainPort: " + joramId + "/" + joramDomainPort
            + " JMX_JNDI: " + jmxJndiPort + " JMX: " + jmxPort
            + " JNDI: " + jndiPort + " HTML: " + htmlPort
            + " DREAM:" + dreamTCPPort
            + " Extra properties:" + properties;
    }

    /**
     * @return the jmxJndiPort
     */
    public String getJmxJndiPort() {
        return jmxJndiPort;
    }

    /**
     * @param jmxJndiPort the jmxJndiPort to set
     */
    public void setJmxJndiPort(String jmxJndiPort) {
        this.jmxJndiPort = jmxJndiPort;
    }
}
