/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.communication.network;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TimerTask;

import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;

import org.objectweb.petals.communication.jmx.rmi.RMIConnector;
import org.objectweb.petals.communication.network.ContainerInformation.ContainerState;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;

/**
 * This PingTask is a service that ping all jndi servers of each petals in
 * function of given period.
 * 
 * @author Nicolas SALATGE - eBMWebSourcing
 * 
 */
public class PingTask extends TimerTask {

	/**
	 * The network service.
	 */
	private NetworkService networkService;

	private Map<String, ContainerState> containerStatesMap;

	/**
	 * the logger.
	 */
	private LoggingUtil log;

	/**
	 * the default constructor.
	 * 
	 * @param ns
	 * 		the network service
	 * @param log
	 * 		the logger
	 */
	public PingTask(final NetworkService ns, final LoggingUtil log,
			final Map<String, ContainerState> containerStatesMap) {
		this.networkService = ns;
		this.log = log;
		this.containerStatesMap = containerStatesMap;
	}

	/**
	 * run the task.
	 */
	@Override
	public void run() {
		final List<ContainerInformation> info = this.networkService
				.retrieveAllContainersInformation(null);

		final Map<String, ContainerState> newContainerStatesMap = new HashMap<String, ContainerState>();

		for (ContainerInformation containerInfo : info) {
			if (!containerInfo.getName().equals(SystemUtil.getContainerName())) {
				if (ContainerState.STARTED.equals(containerInfo.getState())
						&& this.ping(containerInfo.getHost(), containerInfo
								.getJmxPort(), containerInfo.getJmxJndiPort(),
								containerInfo.getJmxLogin(), containerInfo
										.getJmxPassword())) {
					newContainerStatesMap.put(containerInfo.getName(),
							ContainerState.STARTED);
				} else {
					newContainerStatesMap.put(containerInfo.getName(),
							ContainerState.STOPPED);

				}
			}
		}
		// update the container local view
		this.containerStatesMap.putAll(newContainerStatesMap);
		this.containerStatesMap.keySet().retainAll(
				newContainerStatesMap.keySet());
	}

	/**
	 * Realize the ping on a host and a specific port.
	 * 
	 * @param host
	 * 		the host to ping. May be null.
	 * @param port
	 * 		the port to ping
	 * @return true if the server is alive
	 */
	private boolean ping(final String host, final String jmxPort,
			final String jmxJNDIPort, final String jmxUser, final String jmxPwd) {
		this.log.start("Ping of " + host + " on port " + jmxJNDIPort);

		try {
			JMXConnector connector = null;
			Map<String, Object> args = new HashMap<String, Object>();
			if (jmxUser != null) {
				String username = jmxUser;
				String password = jmxPwd;

				String[] credentials = new String[] { username, password };
				args.put("jmx.remote.credentials", credentials);
			}

			String urlStr = RMIConnector.getJmxServiceURL(host, Integer
					.valueOf(jmxPort), Integer.valueOf(jmxJNDIPort));
			JMXServiceURL url = new JMXServiceURL(urlStr);
			connector = JMXConnectorFactory.newJMXConnector(url, args);
			connector.connect();
			connector.close();
		} catch (Exception e) {
			return false;
		}
		return true;
	}

}
