/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: ContainerService.java,v 1.1 2007/04/11 13:34:07 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.container;

import org.objectweb.fractal.api.Component;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.ow2.petals.jbi.descriptor.ComponentDescription;
import org.ow2.petals.jbi.descriptor.ServiceAssembly;
import org.ow2.petals.jbi.descriptor.SharedLibrary;

/**
 * This {@link ContainerService} interface allow to create and handle JBI
 * containers.
 * @author Nicolas SALATGE - eBM WebSourcing
 */
public interface ContainerService {

    /**
     * Create an intaller from the component description of the component.
     * @param componentDescription
     *            the component description of the component
     * @return the fractal component of the installer
     * @throws PetalsException : occurs when it is impossible to create an installer
     */
    Component createInstaller(ComponentDescription componentDescription)
            throws PetalsException;

    /**
     * Remove an installer from the name of the component.
     * @param name
     *            the name of the component
     * @throws PetalsException : occurs when it is impossible to remove an installer
     */
    void removeInstaller(String name) throws PetalsException;

    /**
     * Create a component life cycle from the component description of the component.
     * @param componentDescription
     *            the component description of the component
     * @return the fractal component of the component life cycle
     * @throws PetalsException : occurs when it is impossible to create a component life cycle
     */
    Component createComponentLifeCycle(
            ComponentDescription componentDescription) throws PetalsException;

    /**
     * Remove a component life cycle from the component description of the component.
     * @param componentDescription
     *            the component description of the component
     * @throws PetalsException : occurs when it is impossible to remove a component life cycle
     */
    void removeComponentLifeCycle(
            ComponentDescription componentDescription) throws PetalsException;

    /**
     * Get the container component.
     * @return the parent component (the fractal container)
     */
    Component getParentContainer();

    /**
     * Create a service assembly life cycle.
     * It is realize from the component description of the component.
     * @param sa
     *            the service assembly of the component
     * @return the fractal component of the service assembly life cycle
     * @throws PetalsException : occurs when it is impossible to create a service assembly
     */
    Component createServiceAssemblyLifeCycle(
           ServiceAssembly sa) throws PetalsException;

    /**
     * remove a service assembly life cycle.
     * @param sa
     *            the service assembly of the component
     * @throws PetalsException : occurs when it is impossible to remove a service assembly life cycle
     */
    void removeServiceAssemblyLifeCycle(
           ServiceAssemblyLifeCycle sa) throws PetalsException;


    /**
     * Create a shared library life cycle component.
     * @param sl
     *            the shared library
     * @return the fractal component of the shared library
     * @throws PetalsException : occurs when it is impossible to create a shared library
     */
    Component createSharedLibraryLifeCycle(SharedLibrary sl)
            throws PetalsException;

    /**
     * Remove a shared library life cycle component.
     * @param slName
     *            the shared library name
     * @throws PetalsException : occurs when it is impossible to remove a shared library
     */
    void removeSharedLibraryLifeCycle(String slName) throws PetalsException;

}
