/**
 * 
 */
package org.objectweb.petals.discovery;

import java.util.Hashtable;
import java.util.List;
import java.util.Map;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.communication.jmx.DistributedJMXServer;
import org.objectweb.petals.communication.jmx.JMXService;
import org.objectweb.petals.communication.network.ContainerInformation;
import org.objectweb.petals.communication.network.NetworkService;
import org.objectweb.petals.communication.network.ContainerInformation.ContainerState;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * @author rnaudin
 * 
 */

@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.discovery.DiscoveryMBean.class))
public class DiscoveryService implements DiscoveryMBean {

	@Requires(name = "network", signature = org.objectweb.petals.communication.network.NetworkService.class)
	protected NetworkService networkService;

	@Requires(name = "jmx", signature = org.objectweb.petals.communication.jmx.JMXService.class)
	protected JMXService jmxService;

	/**
	 * Logging utilities
	 */
	protected LoggingUtil log;

	@Monolog(name = "logger")
	protected Logger logger;

	/**
	 * The local container name
	 */
	private String localContainerName;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.petals.communication.discovery.DiscoveryMBean#
	 * notifyContainerState(java.lang.String,
	 * org.objectweb.petals.communication.
	 * network.ContainerInformation.ContainerState)
	 */
	public void notifyContainerState(String containerName, ContainerState state)
			throws PetalsException {
		this.log.call();
		this.networkService.updateContainerState(containerName, state);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.petals.discovery.DiscoveryMBean#getDomainView()
	 */
	public Map<String, String> getDomainView() {
		this.log.call();
		Map<String, String> domainView = new Hashtable<String, String>();
		Map<String, ContainerState> _domainView = this.networkService
				.getNetworkView();
		for (String name : _domainView.keySet()) {
			if (_domainView.get(name) == ContainerState.STARTED) {
				domainView.put(name, "STARTED");
			} else if (_domainView.get(name) == ContainerState.STOPPED) {
				domainView.put(name, "STOPPED");
			} else {
				domainView.put(name, "UNKNOWN");
			}
		}

		return domainView;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.objectweb.petals.discovery.DiscoveryMBean#refreshDomainView()
	 */
	public void refreshDomainView() {
		this.log.call();
		this.networkService.refreshNetworkView();
	}

	/**
	 * Start Fractal component.
	 * 
	 * @throws Exception
	 */
	@LifeCycle(on = LifeCycleType.START)
	protected void start() {
		this.log = new LoggingUtil(logger);
		this.log.start();

		ContainerInformation localContainerInformation = this.networkService
				.getLocalContainerInformation();
		localContainerName = localContainerInformation.getName();

		List<ContainerInformation> containerInformations = this.networkService
				.retrieveAllContainersInformation(ContainerState.STARTED);

		for (ContainerInformation containerInformation : containerInformations) {
			if (containerInformation.getName().equals(localContainerName)) {
				continue;
			}

			DistributedJMXServer distributedJMXServer;
			try {
				distributedJMXServer = this.jmxService
						.createDistributedJMXServer(containerInformation
								.getName());

				distributedJMXServer.invoke(distributedJMXServer
						.getDiscoveryServiceMBeanName(),
						"notifyContainerState", new Object[] {
								localContainerName, ContainerState.STARTED },
						new String[] { String.class.getName(),
								ContainerState.class.getName() });
				
				distributedJMXServer.closeConnector();

			} catch (Exception e) {
				this.log.debug(e);
			}
		}

		this.log.end();

	}

	/**
	 * Stop Fractal component.
	 * 
	 * @throws Exception
	 */
	@LifeCycle(on = LifeCycleType.STOP)
	protected void stop() {
		this.log.start();

		List<ContainerInformation> containerInformations = this.networkService
				.retrieveAllContainersInformation(ContainerState.STARTED);

		for (ContainerInformation containerInformation : containerInformations) {
			try {
				if (containerInformation.getName().equals(localContainerName)
						|| !this.networkService
								.isContainerStarted(containerInformation
										.getName())) {
					continue;
				}

				DistributedJMXServer distributedJMXServer = this.jmxService
						.createDistributedJMXServer(containerInformation
								.getName());

				distributedJMXServer.invoke(distributedJMXServer
						.getDiscoveryServiceMBeanName(),
						"notifyContainerState", new Object[] {
								localContainerName, ContainerState.STOPPED },
						new String[] { String.class.getName(),
								ContainerState.class.getName() });
				
				distributedJMXServer.closeConnector();

			} catch (Exception e) {
				this.log.debug(e);
			}
		}

		this.log.end();
	}

}
