/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: UnregisterSALifeCycleTask.java 154 10 oct. 06 ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.task.deployment.undeploy;

import java.util.HashMap;

import org.objectweb.petals.container.ContainerService;
import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.objectweb.petals.processor.Task;

/**
 * This task removes the service assembly life cycle from the manager service SA
 * Map<br/> Inputs : entity name {@link String}
 * 
 * @author ofabre - EBM Websourcing
 * 
 */
public class UnregisterSALifeCycleTask implements Task {

    /**
     * JBI Container :: JMX Agent Admin
     */
    protected AdminServiceMBean managerService;
    
    /**
     * JBI Container :: Fractal container manager
     */
    protected ContainerService containerService;

    public UnregisterSALifeCycleTask(AdminServiceMBean managerService, ContainerService containerService) {
        super();
        this.managerService = managerService;
        this.containerService = containerService;
    }

    public void execute(HashMap context) throws Exception {
        ServiceAssemblyLifeCycle saLifeCycle = (ServiceAssemblyLifeCycle) context
        .get(DeploymentContextConstants.SA_LIFECYCLE);

        String name = (String) context.get(TaskContextConstants.ENTITY_NAME);

        managerService.unregisterServiceAssembly(name);
        
        /*
         * Remove the fractal component
         */
        containerService.removeServiceAssemblyLifeCycle(saLifeCycle);

    }

    public void undo(HashMap context) throws Exception {
        // Nothing to do

    }

}
