/**
 * PETALS: PETALS Services Platform
 * Copyright (C) 2005 EBM WebSourcing
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA.
 *
 * Initial developer(s): pierre.garcia@inrialpes.fr  
 */

package org.objectweb.petals.jbi.messaging.transport.dream;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;

import org.objectweb.petals.jbi.messaging.exchange.MessageExchangeImpl;

import org.objectweb.dream.message.AbstractChunk;
import org.objectweb.dream.util.Util;

public class MessageExchangeChunk extends AbstractChunk<MessageExchangeChunk> {

    /**
     * UID for serializable class
     */
    private static final long serialVersionUID = 1L;

    /**
     * The default name of chunk. This is used to identify the chunk in the
     * dream message.
     */
    public static final String DEFAULT_NAME = "message-exchange-chunk";

    // ----------------------------------------------------------------------------------------------
    // Content of the chunk
    // ----------------------------------------------------------------------------------------------

    /**
     * The message Exchange contained in the chunk TODO : divide message
     * exchange in multiple chunk. maybe it increases performances.
     */
    private MessageExchangeImpl messageExchange;

    // -------------------------------------------------------------------------
    // Chunk methods implementation
    // -------------------------------------------------------------------------

    /**
     * Get the content of the chunk, i.e. the message exchange. This method must
     * be called after having fetched the related chunk in the dream message.
     * 
     * @return the message exchange contained in the chunk
     */
    public MessageExchangeImpl getMessageExchange() {
        return messageExchange;
    }

    /**
     * Set the content of the chunk with the messageExchange passed as
     * parameter. This method must be called after the creation of the chunk
     * before sending the message.
     * 
     * @param jbi
     *            the message exchange to put in the chunk
     */
    public void setMessageExchange(MessageExchangeImpl jbi) {
        messageExchange = jbi;
    }

    /**
     * @see org.objectweb.dream.message.AbstractChunk#newChunk()
     */
    @Override
    protected MessageExchangeChunk newChunk() {
        return new MessageExchangeChunk();
    }

    /**
     * @see org.objectweb.dream.message.AbstractChunk#transfertStateTo(org.objectweb.dream.message.AbstractChunk)
     */
    @Override
    protected void transfertStateTo(MessageExchangeChunk newInstance) {
        newInstance.setMessageExchange(this.messageExchange);
    }

    // -------------------------------------------------------------------------
    // Implementation of the Externalizable interface
    // -------------------------------------------------------------------------

    /**
     * @see java.io.Externalizable#readExternal(java.io.ObjectInput)
     */
    public void readExternal(ObjectInput in) throws IOException,
            ClassNotFoundException {
            this.messageExchange = (MessageExchangeImpl) Util.readObject(in);
    }

    /**
     * @see java.io.Externalizable#writeExternal(java.io.ObjectOutput)
     */
    public void writeExternal(ObjectOutput out) throws IOException {
            Util.writeObject(out, this.messageExchange);
    }

    // -------------------------------------------------------------------------
    // Implementation of the Recyclable interface
    // -------------------------------------------------------------------------

    /**
     * @see org.objectweb.dream.pool.Recyclable#recycle()
     */
    public void recycle() {
        this.messageExchange = null;
    }
}
