/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.kernel.interceptors.stat;

import java.lang.reflect.Method;

import org.objectweb.fractal.julia.asm.SimpleCodeGenerator;

/**
 * This code generator generates interception code of the following form for all
 * methods.
 *
 * <p>
 *
 * <pre>
 * &lt;i&gt;method-signature&lt;/i&gt; {
 *   &lt;i&gt;return-type&lt;/i&gt; result;
 *   long context = delegate.preMethod(&lt;i&gt;method-name&lt;/i&gt;);
 *   // original method code, where returns are replaced with gotos
 *   delegate.postMethod(&lt;i&gt;method-name&lt;/i&gt;, context);
 *   return result;
 * }
 * </pre>
 *
 * where <tt>delegate</tt> is initialized with the following code:
 *
 * <p>
 *
 * <pre>
 * delegate = (...)ic.getInterface(&quot;stat-controller&quot;);
 * </pre>
 */
public class StatCodeGenerator extends SimpleCodeGenerator {

    /**
     * Default constructor.
     */
    public StatCodeGenerator() {

    }

    /**
     * Get the controller interface name.
     *
     * @return the controller interface name.
     */
    protected String getControllerInterfaceName() {
        return "stat-controller";
    }

    /**
     * get the pre method name.
     *
     * @return the pre method name
     */
    protected String getPreMethodName() {
        return "preMethod";
    }

    /**
     * get the post method name.
     *
     * @return the post method name
     */
    protected String getPostMethodName() {
        return "postMethod";
    }

    /**
     * get the context type.
     *
     * @return the context type
     */
    protected Class getContextType() {
        return String.class;
    }

    /**
     * Get the method name.
     *
     * @param m
     *            the method
     * @return the method name
     */
    protected String getMethodName(final Method m) {
        String separator = "_";
        return m.getDeclaringClass() + separator
                + m.toGenericString();
    }
}
