/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: SystemUtil.java 446 2006-05-19 14:13:50Z ddesjardins $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.util;

import java.io.File;
import java.net.SocketException;
import java.net.URI;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import org.objectweb.petals.PetalsException;
import org.objectweb.petals.jbi.messaging.transport.Transporter;
import org.objectweb.petals.jbi.messaging.transport.dream.DreamWrapper;

/**
 * This class is used to store the Petals configuration
 * 
 * @version $Rev: 446 $ $Date: 2006-05-19 16:13:50 +0200 (ven, 19 mai 2006) $
 * @since Petals 1.0
 * @author alouis, Rafael Marins
 * @author ddesjardins - eBMWebsourcing
 * @author Marc Dutoo - Open Wide
 * @author chamerling - eBMWebsourcing
 */
public final class SystemUtil {

	/* properties keys */

	public static final String CONTAINER_NAME = "container.name";

	public static final String CONTAINER_UID = "container.uid";

	public static final String HOST = "host";
	
	public static final String HTML_PORT = "html.port";

	public static final String INSTALL_DIRECTORY = "install.directory";

	public static final String JMX_JNDI_PORT = "jmx.jndi.port";

	public static final String JMX_PORT = "jmx.port";

	public static final String JMX_PWD = "jmx.pwd";

	public static final String JMX_USER = "jmx.user";

	public static final String JNDI_FACTORY = "java.naming.factory.initial";
	
	public static final String JNDI_CLIENT = "java.naming.client";

    public static final String JNDI_PORT = "java.naming.factory.port";

	public static final String JNDI_SERVER = "jndi.server";

	public static final String JORAM_DOMAIN = "joram.domain";

	public static final String JORAM_DOMAIN_PORT = "joram.domainport";

	public static final String JORAM_ID = "joram.id";

	public static final String JORAM_PWD = "joram.pwd";

	public static final String JORAM_TCP_PORT = "joram.tcpport";

	public static final String JORAM_USER = "joram.user";

	public static final String DREAM_TCP_PORT = DreamWrapper.PROP_NAME_LISTEN_PORT;

	public static final String SSL_IDENTITYKEYSTORE = "server.identitykeystore";

	public static final String SSL_IDENTITYKEYSTORE_PWD = "server.identitykeystore.password";

	public static final String SSL_TRUSTEDKEYSTORE = "server.trustedkeystore";

	public static final String SSL_TRUSTEDKEYSTORE_PWD = "server.trustedkeystore.password";

	public static final String TASK_TIMEOUT = "task.timeout";

	public static final String TRANSPORT_QOS = "transport.qos";

	public static final String TRIBE_MULTICAST = "tribe.multicast";

	public static final String VALID_EXCHANGE = "exchange.validation";

	public static final String REPOSITORY_PATH = "repository.path";

	public static final String JORAM_PATH = "joram.path";

	public static final String ROUTING_STRATEGY = "routing.strategy";

	public static final String PING_PERIOD = "ping.period";

	public static final String CACHE_ACTIVATED = "cache.activated";

	public static final String CACHE_CLEAN_PERIOD = "cache.clean.period";

	public static final String NETWORK_TIME_SYNCHRONIZED = "network.time.synchronized";

	public static final String CONTAINER_HA = "container.ha";
	
	public static final String RECOVERING_WORKERS = "recovering.workers";
	
	public static final String FAST_TRANSPORTER = "transporter.fast";
	
	/**
	 * The system properties
	 */
	private static Map<String, String> global = new HashMap<String, String>();
	static {
		global = PropertyUtil.loadGlobalProperties();
	}

	/**
	 * A map of updated values
	 */
	private static Map<String, String> updated = new HashMap<String, String>();
	static {
		updated = PropertyUtil.loadUpdatedProperties();
	}

	/**
	 * A map of default values
	 */
	private static Map<String, String> defaultValues = new HashMap<String, String>();
	static {
		defaultValues.put(CONTAINER_NAME, "0");
		defaultValues.put(CONTAINER_UID, "0");
		defaultValues.put(DREAM_TCP_PORT, DreamWrapper.DEFAULT_LISTEN_PORT);
		defaultValues.put(HOST, "127.0.0.1");
		defaultValues.put(HTML_PORT, "7080");
		defaultValues.put(JMX_JNDI_PORT, "7700");
		defaultValues.put(JMX_PORT, "-1"); // Dynamically setted
		defaultValues.put(JMX_PWD, "");
		defaultValues.put(JMX_USER, "");
		defaultValues
		        .put(JNDI_FACTORY,
                        "org.objectweb.petals.communication.jndi.tribe.connection.NamingContextFactory");
		defaultValues
		        .put(JNDI_CLIENT,
                        "org.objectweb.petals.communication.jndi.JNDIServiceImpl");
		defaultValues.put(JNDI_PORT, "7720");
		defaultValues.put(JNDI_SERVER, "tribe");
		defaultValues.put(JORAM_DOMAIN, "eBM");
		defaultValues.put(JORAM_DOMAIN_PORT, "7740");
		defaultValues.put(JORAM_ID, "0");
		defaultValues.put(JORAM_PWD, "petals");
		defaultValues.put(JORAM_TCP_PORT, "7760");
		defaultValues.put(JORAM_USER, "petals");
		defaultValues.put(REPOSITORY_PATH, "");
		defaultValues.put(JORAM_PATH, "");
		defaultValues.put(SSL_IDENTITYKEYSTORE, "");
		defaultValues.put(SSL_IDENTITYKEYSTORE_PWD, "");
		defaultValues.put(SSL_TRUSTEDKEYSTORE, "");
		defaultValues.put(SSL_TRUSTEDKEYSTORE_PWD, "");
		defaultValues.put(TASK_TIMEOUT, "0");
		defaultValues.put(TRANSPORT_QOS, Transporter.DEFAULT_POLICY);
		defaultValues.put(TRIBE_MULTICAST, "224.7.65.23");
		defaultValues.put(VALID_EXCHANGE, "false");
		defaultValues.put(ROUTING_STRATEGY, "");
		defaultValues.put(PING_PERIOD, "300000");
		defaultValues.put(CACHE_ACTIVATED, "true");
		defaultValues.put(CACHE_CLEAN_PERIOD, "300000");
		defaultValues.put(NETWORK_TIME_SYNCHRONIZED, "false");
		defaultValues.put(CONTAINER_HA, "true");
        defaultValues.put(RECOVERING_WORKERS, "1");
        defaultValues.put(FAST_TRANSPORTER, "DreamTransportProtocol");
	}

	/**
	 * Creates a new instance of {@link SystemUtil}
	 * 
	 */
	private SystemUtil() {
		super();
	}

	/**
	 * 
	 * @param key
	 * @return
	 */
	public static final String getValue(String key) {
		return global.get(key);
	}

	/**
	 * 
	 * @param key
	 * @param value
	 */
	private static final void setAndUpdateValue(String key, String value) {
		String old = getValue(key);
		if (old == null || !old.equals(value)) {
			updated.put(key, value);
		}
		global.put(key, value);
	}

	/**
	 * Get the value from the config file. If the value is not found, set the
	 * default one and update the updated values map.
	 * 
	 * @param key
	 * @return
	 */
	public static String getAndUpdateValue(String key) {
		if (getValue(key) == null) {
			setAndUpdateValue(key, getDefault(key));
		}
		return getValue(key);
	}

	/**
	 * Get the default value
	 * 
	 * @param key
	 * @return
	 */
	private static String getDefault(String key) {
		String result = defaultValues.get(key);
		if (result == null) {
			result = "";
		}
		return result;
	}

	/**
	 * Get the container name.
	 * 
	 * @return
	 */
	public static String getContainerName() {
		return getAndUpdateValue(CONTAINER_NAME);
	}

	/**
	 * 
	 * @return
	 */
	public static long getContainerUID() {
		String containerUID = getAndUpdateValue(CONTAINER_UID);
		if (containerUID.equals(getDefault(CONTAINER_UID))) {
			containerUID = Long.toString(System.currentTimeMillis());
		}
		setAndUpdateValue(CONTAINER_UID, containerUID);
		return Long.parseLong(getValue(CONTAINER_UID));
	}

	/**
	 * Get the Dream TCP Port
	 * 
	 * @return
	 */
	public static String getDreamTCPPort() {
		return getAndUpdateValue(DREAM_TCP_PORT);
	}

	/**
	 * Get the fast transporter name
	 * 
	 * @return
	 */
	public static String getFastTransporter() {
		return getAndUpdateValue(FAST_TRANSPORTER);
	}

	/**
	 * 
	 * @return
	 */
	public static String getHost() {
		String host = getAndUpdateValue(HOST);
		if (host.equals(getDefault(HOST))) {
			host = resolveLocalIP();
		}
		setAndUpdateValue(HOST, host);
		return getValue(HOST);
	}

	/**
	 * 
	 * @return
	 */
	public static String getHtmlPort() {
		return getAndUpdateValue(HTML_PORT);
	}

	/**
	 * 
	 * @return
	 */
	public static String getIdentitykeystore() {
		return getAndUpdateValue(SSL_IDENTITYKEYSTORE);
	}

	/**
	 * 
	 * @return
	 */
	public static String getIdentitykeystorePassword() {
		return getAndUpdateValue(SSL_IDENTITYKEYSTORE_PWD);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJmxPassword() {
		return getAndUpdateValue(JMX_PWD);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJmxJndiPort() {
		return getAndUpdateValue(JMX_JNDI_PORT);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJmxPort() {
		if (getValue(JMX_PORT) == null || "".equals(getValue(JMX_PORT))) {
			setAndUpdateValue(JMX_PORT, getDefault(JMX_PORT));
		}
		return getValue(JMX_PORT);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJmxUser() {
		return getAndUpdateValue(JMX_USER);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJndiFactory() {
		return getAndUpdateValue(JNDI_FACTORY);
	}

	/**
     * 
     * @return
     */
    public static String getJndiClient() {
        return getAndUpdateValue(JNDI_CLIENT);
    }

    /**
     * 
     * @return
     */
    public static String getJndiPort() {
        return getAndUpdateValue(JNDI_PORT);
    }

    /**
	 * Get the JNDI server to be used
	 * 
	 * @return the jndi server that is used. Default is tribe.
	 */
	public static String getJNDIServer() {
		return getAndUpdateValue(JNDI_SERVER);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJoramDomain() {
		return getAndUpdateValue(JORAM_DOMAIN);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJoramDomainPort() {
		return getAndUpdateValue(JORAM_DOMAIN_PORT);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJoramId() {
		return getAndUpdateValue(JORAM_ID);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJoramPassword() {
		return getAndUpdateValue(JORAM_PWD);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJoramTCPPort() {
		return getAndUpdateValue(JORAM_TCP_PORT);
	}

	/**
	 * 
	 * @return
	 */
	public static String getJoramUser() {
		return getAndUpdateValue(JORAM_USER);
	}

	/**
	 * Find the petals install directory
	 * 
	 * @return petals install directory
	 * @throws RuntimeException
	 *             PetalsInstallDirectory NotFound
	 */
	public static File getPetalsInstallDirectory() {
		String petalsInstallDirectory = getValue(INSTALL_DIRECTORY);
		File result = null;

		if (petalsInstallDirectory == null) {
			// first try to resolve install directory set in the environment
			File dir = resolveInstallDirectoryFromEnvironment();
			// otherwise, guess install directory using the framework

			if (dir == null) {
				dir = resolveInstallDirectoryFromFramework();
			}
			result = dir;
			global.put(INSTALL_DIRECTORY, result.getAbsolutePath());
		} else {
			result = new File(petalsInstallDirectory);
		}

		if (result == null) {
			throw new RuntimeException(
					"Petals installation directory can not be found. "
							+ "Set a PETALS_HOME variable or use -Dpetals.home system variable.");
		}

		return result;
	}

	/**
	 * Get the transporter Quality Of Service policy
	 * 
	 * @return
	 */
	public static String getTransporterQOS() {
		return getAndUpdateValue(TRANSPORT_QOS);
	}

	/**
	 * 
	 * @return
	 */
	public static String getTribeMulticastAddress() {
		return getAndUpdateValue(TRIBE_MULTICAST);
	}

	/**
	 * 
	 * @return
	 */
	public static String getTrustedkeystore() {
		return getAndUpdateValue(SSL_TRUSTEDKEYSTORE);
	}

	/**
	 * 
	 * @return
	 */
	public static String getTrustedkeystorePassword() {
		return getAndUpdateValue(SSL_IDENTITYKEYSTORE_PWD);
	}

	/**
	 * Get the routing strategy
	 * 
	 * @return
	 */
	public static String getRoutingStrategy() {
		return getAndUpdateValue(ROUTING_STRATEGY);
	}

	/**
	 * Get the Domain ping period
	 * 
	 * @return
	 */
	public static String getPingPeriod() {
		return getAndUpdateValue(PING_PERIOD);
	}

	/**
	 * Get the network synchronized time
	 * 
	 * @return
	 */
	public static String getSynchronizedTime() {
		return getAndUpdateValue(NETWORK_TIME_SYNCHRONIZED);
	}

	/**
	 * Get the HA flag.
	 * 
	 * @return
	 */
	public static String getHA() {
		return getAndUpdateValue(CONTAINER_HA);
	}

	/**
     * Get the worker number of the recovering process
     * 
     * @return
     */
    public static int getRecoveringWorker() {
        int iRet = 1;
        try {
            iRet = Integer.parseInt(getAndUpdateValue(RECOVERING_WORKERS));
            iRet = (iRet == 0 ? 1 : iRet);
        } catch (final NumberFormatException e) {
            // NOP
        }
        
        return iRet;
    }

    /**
	 * Verify if the cache is activated
	 * 
	 * @return
	 */
	public static boolean isCacheActivated() {
		return Boolean.parseBoolean(getAndUpdateValue(CACHE_ACTIVATED));
	}

	/**
	 * Get the cache clean period
	 * 
	 * @return
	 */
	public static String getCacheCleanPeriod() {
		return getAndUpdateValue(CACHE_CLEAN_PERIOD);
	}

	/**
	 * Set the cache clean period
	 * 
	 * @return
	 */
	public static void setCacheCleanPeriod(final String period) {
		setAndUpdateValue(CACHE_CLEAN_PERIOD, period);
	}

	/**
	 * Get the topology mode of the container network.
	 * 
	 * @return
	 */
	public static boolean isDynamicTopology() {
		return "tribe".equalsIgnoreCase(getJNDIServer());
	}

	/**
	 * 
	 * @return
	 */
	public static boolean isExchangeValidation() {
		return Boolean.parseBoolean(getAndUpdateValue(VALID_EXCHANGE));
	}

	public static void setContainerName(String containerName) {
		setAndUpdateValue(CONTAINER_NAME, containerName);
	}

	public static void setContainerUID(long containerUID) {
		setAndUpdateValue(CONTAINER_UID, Long.toString(containerUID));
	}

	public static void setDreamTCPPort(String dreamTCPPort) {
		setAndUpdateValue(DREAM_TCP_PORT, dreamTCPPort);
	}

	public static void setFastTransporter(String fastTransporter) {
		setAndUpdateValue(FAST_TRANSPORTER, fastTransporter);
	}

	public static void setExchangeValidation(String exchangeValidation) {
		setAndUpdateValue(VALID_EXCHANGE, exchangeValidation);
	}

	public static void setHost(String host) {
		setAndUpdateValue(HOST, host);
	}

	public static void setHtmlPort(String htmlPort) {
		setAndUpdateValue(HTML_PORT, htmlPort);
	}

	public static void setIdentitykeystore(String identitykeystore) {
		setAndUpdateValue(SSL_IDENTITYKEYSTORE, identitykeystore);
	}

	public static void setIdentitykeystorePassword(
			String identitykeystorePassword) {
		setAndUpdateValue(SSL_IDENTITYKEYSTORE_PWD, identitykeystorePassword);
	}

	public static void setJmxPassword(String jmxPassword) {
		setAndUpdateValue(JMX_PWD, jmxPassword);
	}

	public static void setJmxJndiPort(String jmxJnidPort) {
		setAndUpdateValue(JMX_JNDI_PORT, jmxJnidPort);
	}

	public static void setJmxPort(String jmxPort) {
		setAndUpdateValue(JMX_PORT, jmxPort);
	}

	public static void setJmxUser(String jmxUser) {
		setAndUpdateValue(JMX_USER, jmxUser);
	}

	public static void setJndiFactory(String jndiFactory) {
		setAndUpdateValue(JNDI_FACTORY, jndiFactory);
	}

	public static void setJndiClient(String jndiClient) {
        setAndUpdateValue(JNDI_CLIENT, jndiClient);
    }

    public static void setJndiPort(String jndiPort) {
        setAndUpdateValue(JNDI_PORT, jndiPort);
    }

    public static void setJoramDomain(String joramDomain) {
		setAndUpdateValue(JORAM_DOMAIN, joramDomain);
	}

	public static void setJoramDomainPort(String joramPort) {
		setAndUpdateValue(JORAM_DOMAIN_PORT, joramPort);
	}

	public static void setJoramId(String joramId) {
		setAndUpdateValue(JORAM_ID, joramId);
	}

	public static void setJoramPassword(String joramPassword) {
		setAndUpdateValue(JORAM_PWD, joramPassword);
	}

	public static void setJoramTCPPort(String joramTCPPort) {
		setAndUpdateValue(JORAM_TCP_PORT, joramTCPPort);

	}

	public static void setJoramUser(String joramUser) {
		setAndUpdateValue(JORAM_USER, joramUser);
	}

	public static void setTransporterQOS(String transporterQOS) {
		setAndUpdateValue(TRANSPORT_QOS, transporterQOS);
	}

	public static void setTribeMulticastAddress(String tribeMulticastAddress) {
		setAndUpdateValue(TRIBE_MULTICAST, tribeMulticastAddress);
	}

	public static void setTrustedkeystore(String trustedkeystore) {
		setAndUpdateValue(SSL_TRUSTEDKEYSTORE, trustedkeystore);
	}

	public static void setTrustedkeystorePassword(String trustedkeystorePassword) {
		setAndUpdateValue(SSL_TRUSTEDKEYSTORE_PWD, trustedkeystorePassword);
	}

	public static void setRepositoryDirectory(String repositoryDirectory) {
		setAndUpdateValue(REPOSITORY_PATH, repositoryDirectory);
	}

	public static void setJoramDirectory(String joramDirectory) {
		setAndUpdateValue(JORAM_PATH, joramDirectory);
	}

	public static void setPetalsInstallDirectory(String petalsInstallDirectory) {
		setAndUpdateValue(INSTALL_DIRECTORY, petalsInstallDirectory);
	}

	/**
	 * set the routing strategy
	 * 
	 * @param strategy
	 *            the strategy
	 */
	public static void setRoutingStrategy(String strategy) {
		setAndUpdateValue(ROUTING_STRATEGY, strategy);
	}

	public static void setPingPeriod(String pingPeriod) {
		setAndUpdateValue(PING_PERIOD, pingPeriod);
	}

	public static void setSynchronizedTime(String synchronizedTime) {
		setAndUpdateValue(NETWORK_TIME_SYNCHRONIZED, synchronizedTime);
	}

	public static void setHA(String haActivated) {
		setAndUpdateValue(CONTAINER_HA, haActivated);
	}

	public static void  setRecoveringWorker(int recoveringWorker) {
	    setAndUpdateValue(RECOVERING_WORKERS, Integer.toString(recoveringWorker));
    }

    /**
	 * Try to get the petals'home environment variable : petals.home system
	 * property, or PETALS_HOME env variable
	 * 
	 * @return the related directory, or null
	 */
	protected static File resolveInstallDirectoryFromEnvironment() {
		File directory = null;
		String petalsHome = System.getProperty("petals.home");
		if (petalsHome == null || petalsHome.trim().equals("")) {
			petalsHome = System.getenv("PETALS_HOME");
		}
		// not null or empty
		if (petalsHome != null && !petalsHome.trim().equals("")) {
			File theHome = new File(petalsHome);
			// exists and it's a directory
			if (theHome.exists() || theHome.isDirectory()) {
				directory = theHome;
			}
		}
		return directory;
	}

	/**
	 * Try to find the current install directory
	 * 
	 * @return the directory or null if a problem occured
	 */
	protected static File resolveInstallDirectoryFromFramework() {
		// guess from the location of the current JAR
		File directory = null;
		URL url = SystemUtil.class.getProtectionDomain().getCodeSource()
				.getLocation();
		if (url != null) {
			try {
				URI baseURI = new URI(url.toString()).resolve("..");
				directory = new File(baseURI);
			} catch (Exception ignored) {
				ignored.printStackTrace();
			}
		}
		return directory;
	}

	/**
	 * Resolve the "localhost" IP address to the real IP First, use
	 * InetAddress.getLocalHost(). If the returned IP is a "127.0.xxx" like IP,
	 * call the system command "ipconfig" or "ifconfig" and parse the result to
	 * extract the IP.
	 * 
	 * @return the real IP address of the machine, or 127.0.0.1 if the
	 *         resolution failed.
	 */
	protected static String resolveLocalIP() {
		String hostIP = "";
		try {
			hostIP = NetworkUtil.getIPv4Address();
		} catch (SocketException e1) {
			hostIP = NetworkUtil.IPV4_LOOPBACK_ADDRESS;
		}
		return hostIP;
	}

	/**
	 * Get the default properties of the system.
	 * 
	 * @return
	 */
	public static Properties getDefaultProperties() {
		Properties props = new Properties();
		props.putAll(defaultValues);
		return props;
	}

	/**
	 * Get the Repository path.
	 */
	public static String getSpecifiedRepositoryPath() {
		String result = null;
		String tmp = getAndUpdateValue(REPOSITORY_PATH);
		if (!"".equals(tmp)) {
			result = tmp;
		}
		return result;
	}

	/**
	 * Get the JORAM path.
	 * 
	 */
	public static String getJoramPath() {
		String result = null;
		String tmp = getAndUpdateValue(JORAM_PATH);

		if (!"".equals(tmp)) {
			result = tmp;
		} else {
			result = new File(getPetalsInstallDirectory(), "joram")
					.getAbsolutePath();
		}
		return result;
	}

	/**
	 * 
	 * @return
	 */
	public static long getTaskTimeout() {
		String s_timeout = getAndUpdateValue(TASK_TIMEOUT);
		long timeout = 0L;
		try {
			timeout = Long.parseLong(s_timeout);
		} catch (NumberFormatException e) {
			timeout = 0L;
		}
		return timeout;
	}

	/**
	 * Return the local system information
	 * 
	 * @return
	 */
	public static final String info() {
		StringBuffer buffer = new StringBuffer("Local container properties :\n");
		Iterator<String> it = global.keySet().iterator();
		while (it.hasNext()) {
			Object key = it.next();
			buffer.append("\t");
			buffer.append(key.toString());
			buffer.append(" : \n\t\t");
			buffer.append(global.get(key));
			buffer.append("\n");
		}
		return buffer.toString();
	}

	/**
	 * Persists the properties
	 * 
	 */
	public static final void persist() {
		// save the updated properties
		Properties updatedValues = new Properties();
		updatedValues.putAll(updated);
		File outDir = new File(getPetalsInstallDirectory(), "conf");
		File file = new File(outDir, PropertyUtil.UPDATED_PROPS);
		try {
			PropertyUtil.saveProperties(updatedValues, file);
		} catch (PetalsException e) {
		}
	}
}
